// -*- c++ -*-
// Generated by assa-genesis
//------------------------------------------------------------------------------
// $Id: DeckManager.cpp,v 1.20 2004/06/05 04:06:54 vlg Exp $
//------------------------------------------------------------------------------
//                            DeckManager.cpp
//------------------------------------------------------------------------------
//  Copyright (c) 2004 by Vladislav Grinchenko 
//
//  This program is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License   
//  as published by the Free Software Foundation; either version  
//  2 of the License, or (at your option) any later version.      
//------------------------------------------------------------------------------
//
// Date   : Tue Feb  3 13:44:27 EST 2004
//
//------------------------------------------------------------------------------

#include <sstream>

#include <gtkmm/fileselection.h>

#include "Granule-main.h"
#include "Granule.h"
#include "Deck.h"
#include "DeckPlayer.h"
#include "DeckManager.h"
#include "DeckList.h"
#include "MainWindow.h"
#include "Config.h"
#include "CardRef.h"
#include "CardBox.h"

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//                          Toolbar callbacks
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

DeckManager::
DeckManager () : m_untitled_count (1)
{
	trace_with_mask("DeckManager::DeckManager", GUITRACE);
}

DeckManager::
~DeckManager ()
{
	trace_with_mask("DeckManager::~DeckManager", GUITRACE);
}

void
DeckManager::
new_deck_cb ()
{
	trace_with_mask("DeckManager::new_deck_cb", GUITRACE);

	std::ostringstream os;
	os << "Untitled" << m_untitled_count++ << DECK_FILE_EXT;
	Deck* deck = new Deck (os.str (), NEW_DECK);
	on_deck_loaded (deck, LOAD_OK);
}

void
DeckManager::
play_deck_cb ()
{
	trace_with_mask("DeckManager::play_deck_cb", GUITRACE);
	
	Deck* selected = DECKLIST->get_selected ();

	if (selected != NULL) {
		DeckPlayerVectorIter i = m_players.begin ();
		while (i != m_players.end ()) {
			if ((*i)->get_deck () == selected) {
				(*i)->run ();
				DECKLIST->unselect ();
				break;
			}
			i++;
		}
	}
	else {
		Gtk::MessageDialog w ("Select a deck from the list first", 
							  MESSAGE_WARNING);
		w.run ();
	}
}

/** If deck is not loaded, try to load the deck first.
	Search it for the card in the deck and return it if found.
*/
CardRef*
DeckManager::
lookup_card (long id_, const string& path_)
{
	trace_with_mask("DeckManager::lookup_card", GUITRACE);
	Deck* deck;
	VCard* card;

	DeckList::iterator iter = DECKLIST->begin ();
	while (iter != DECKLIST->end ()) {
		deck = DECKLIST->get_deck (iter);
		if (deck->get_name () == path_) {
			break;
		}
		iter++;
	}
	/** Deck is not found 
	 */
	if (iter == DECKLIST->end ()) {
		deck = new Deck (INVALID_DECK);
		deck->load (path_);
	}
	
	if ((card = deck->find_card_by_id (id_)) != NULL) {
		return (new CardRef (*card, *deck));
	}
	return (NULL);
}

void
DeckManager::
open_deck_cb ()
{
	trace_with_mask("DeckManager::open_deck_cb", GUITRACE);

	Deck* deck = new Deck (INVALID_DECK);
	Gtk::FileSelection f ("Choose a deck");
	f.set_filename (CONFIG->get_filesel_path ());
	f.complete ("*" DECK_FILE_EXT);
	f.hide_fileop_buttons ();
	int ret = f.run ();
	if (ret == Gtk::RESPONSE_OK) {
		deck->load (f.get_filename ());
		CONFIG->set_filesel_path (f.get_filename ());
	}
}

/** Insert cards from a deck to the one currently selected.
 */
void
DeckManager::
insert_deck_cb ()
{
	trace_with_mask("DeckManager::insert_deck_cb", GUITRACE);

	Deck* selected = DECKLIST->get_selected ();
	if (selected == NULL) {
		Gtk::MessageDialog w ("Select a deck from the list first", 
							  MESSAGE_WARNING);
		w.run ();
	}
	else {
		Gtk::FileSelection f ("Choose a Deck to Insert From");
		f.set_filename (CONFIG->get_filesel_path ());
		f.complete ("*" DECK_FILE_EXT);
		f.hide_fileop_buttons ();

		if (f.run () == Gtk::RESPONSE_OK) {
			selected->insert (f.get_filename ());
			CONFIG->set_filesel_path (f.get_filename ());
			DeckPlayerVectorIter iter = m_players.begin ();
			while (iter != m_players.end ()) {
				if (selected == dynamic_cast<Deck*>((*iter)->get_deck ())) {
					m_players.erase (iter);
					break;
				}
				iter++;
			}
			m_players.push_back (new DeckPlayer (*selected));
		}
	}
}

void
DeckManager::
save_deck_cb ()
{
	trace_with_mask("DeckManager::save_deck_cb", GUITRACE);
	Deck* selected = DECKLIST->get_selected ();
	if (selected != NULL) {
		if (selected->needs_renaming ()) {
			save_as_deck_cb ();
		}
		else {
			selected->save ();
		}
	}
	else {
		Gtk::MessageDialog w ("Select a deck from the list first", 
							  MESSAGE_WARNING);
		w.run ();
	}
}

void
DeckManager::
save_all_deck_cb ()
{
	trace_with_mask("DeckManager::save_all_deck_cb", GUITRACE);

	bool save_all = false;
	DeckEditStatus deck_status;
	DeckList::iterator iter = DECKLIST->begin ();

	DL((GRAPP,"Decks in DeckList = %d\n", DECKLIST->size ()));

	while (iter != DECKLIST->end ()) {
		if (DECKLIST->deck_needs_renaming (iter)) {
			Gtk::FileSelection f ("Save New Deck");
			f.set_filename (CONFIG->get_filesel_path ());
			f.complete ("*" DECK_FILE_EXT);
			f.hide_fileop_buttons ();
			if (f.run () == Gtk::RESPONSE_OK) {
				DECKLIST->get_deck (iter)->save_as (f.get_filename ());
				CONFIG->set_filesel_path (f.get_filename ());
			}
		}
		else {
			if (DECKLIST->get_deck_status (iter) == MODIFIED_DECK) {
				if (save_all) {
					DECKLIST->save (iter);
					goto next;
				}
				Gtk::MessageDialog qm (*MAINWIN,
								   "Deck \"" + DECKLIST->get_name (iter) + 
						   "\" has been modified.\nWould you like to save it?",
								   Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, 
								   true, true);
				qm.add_button ("Save", Gtk::RESPONSE_OK);
				qm.add_button ("Save All", Gtk::RESPONSE_YES);
				qm.add_button ("Don't Save", Gtk::RESPONSE_NO);
				int ret = qm.run ();
				if (ret == Gtk::RESPONSE_OK || ret == Gtk::RESPONSE_YES) {
					DECKLIST->save (iter);
					if (ret == Gtk::RESPONSE_YES) {
						save_all = true;
					}
				}
			}
		}
	next:
		iter++;
	}
}

void
DeckManager::
save_as_deck_cb ()
{
	trace_with_mask("DeckManager::save_as_deck_cb", GUITRACE);

	Deck* selected = DECKLIST->get_selected ();

	if (selected != NULL) {
		Gtk::FileSelection f ("Save Deck As");
		f.set_filename (CONFIG->get_filesel_path ());
		f.complete ("*" DECK_FILE_EXT);

		if (f.run () == Gtk::RESPONSE_OK) {
			selected->save_as (f.get_filename ());
			DECKLIST->refresh_selected_name ();
			CONFIG->set_filesel_path (f.get_filename ());
		}
	}
	else {
		Gtk::MessageDialog w ("Select a deck from the list first", 
							  MESSAGE_WARNING);
		w.run ();
	}
}

void
DeckManager::
add_deck_to_box  (int idx_)
{
	trace_with_mask("DeckManager::add_deck_to_box", GUITRACE);
	Deck* selected = DECKLIST->get_selected ();
	if (selected) {
		CARDBOX->add_deck_to_box (idx_, selected);
	}
	else {
		Gtk::MessageDialog w ("Select a deck from the list first", 
							  MESSAGE_WARNING);
		w.run ();
	}
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//                          Deck management functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

void 
DeckManager::
on_deck_loaded (Deck* deck_, DeckLoadStatus status_)
{
	trace_with_mask("DeckManager::on_deck_loaded", GUITRACE);

	if (status_ == LOAD_FAILED) {
		delete deck_;
		Gtk::MessageDialog emsg ("Failed to load deck file!\n"
							 "Check file/DTD syntax with \"xmllint <fname>\"",
								 MESSAGE_ERROR);
		emsg.run ();
	}
	else {
		DECKLIST->push_back (deck_);
		m_players.push_back (new DeckPlayer (*deck_));
	}
}

